Package: plyxp
Title: Data masks for SummarizedExperiment enabling dplyr-like
        manipulation
Version: 1.7.3
Authors@R: c(
    person("Justin", "Landis", email = "jtlandis314@gmail.com", role = c("aut", "cre"),
    comment = c(ORCID = "0000-0001-5501-4934")),
    person("Michael", "Love", email = "michaelisaiahlove@gmail.com", role = "aut",
    comment = c(ORCID = "0000-0001-8401-0545")))
Description: The package provides `rlang` data masks for the SummarizedExperiment class. The enables the evaluation of unquoted expression in different contexts of the SummarizedExperiment object with optional access to other contexts. The goal for `plyxp` is for evaluation to feel like a data.frame object without ever needing to unwind to a rectangular data.frame.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Imports: dplyr, purrr, rlang, SummarizedExperiment, tidyr, tidyselect,
        vctrs, tibble, pillar, cli, glue, S7, S4Vectors, utils, methods
Depends: R (>= 4.5.0)
Suggests: devtools, knitr, rmarkdown, testthat, airway, IRanges, here
LazyData: true
URL: https://github.com/jtlandis/plyxp,
        https://jtlandis.github.io/plyxp
BugReports: https://www.github.com/jtlandis/plyxp/issues
biocViews: Annotation, GenomeAnnotation, Transcriptomics
Config/testthat/edition: 3
Collate: 'as.data.frame.R' 'connect_masks.R' 'context-utils.R' 'data.R'
        'dplyr-across.R' 'dplyr-arrange.R' 'dplyr-filter.R'
        'dplyr-group_by.R' 'dplyr-mutate.R' 'dplyr-pull.R'
        'dplyr-select.R' 'dplyr-slice.R' 'dplyr-summarise.R'
        'group_utils.R' 'grouped_list.R' 'vctrs-S4-.R' 'vctrs-S4-rep.R'
        'vctrs-S4-unchop.R' 'vctrs-S4-chop.R' 'mask_env_top.R'
        'plyxp-R6-manager.R' 'plyxp-R6.R' 'plyxp-class.R'
        'plyxp-package.R' 'plyxp-quos.R' 'plyxp.R' 'plyxp_cannot.R'
        'print.R' 'reexports-SummarizedExperiment.R'
        'reexports-dplyr.R' 'utils.R' 'vctrs-S4-group.R'
        'vctrs-S4-recycle.R' 'vctrs-S4-slice.R' 'zzz-globals.R' 'zzz.R'
git_url: https://git.bioconductor.org/packages/plyxp
git_branch: devel
git_last_commit: ed59c03
git_last_commit_date: 2026-07-28
Repository: Bioconductor 3.24
Date/Publication: 2026-07-29
NeedsCompilation: no
Packaged: 2026-07-29 23:29:30 UTC; biocbuild
Author: Justin Landis [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5501-4934>),
  Michael Love [aut] (ORCID: <https://orcid.org/0000-0001-8401-0545>)
Maintainer: Justin Landis <jtlandis314@gmail.com>
