# The following functions should
# be available to users from our package

#' @export
SummarizedExperiment::SummarizedExperiment

#' @export
SummarizedExperiment::assay

#' @export
SummarizedExperiment::`assay<-`

#' @export
SummarizedExperiment::assays

#' @export
SummarizedExperiment::`assays<-`

#' @export
SummarizedExperiment::rowData

#' @export
SummarizedExperiment::`rowData<-`

#' @export
SummarizedExperiment::colData

#' @export
SummarizedExperiment::`colData<-`

#' @export
S4Vectors::metadata

#' @export
S4Vectors::`metadata<-`
