% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.R
\name{as.data.frame}
\alias{as.data.frame}
\alias{as.data.frame.PlySummarizedExperiment}
\title{create data.frame}
\usage{
\method{as.data.frame}{PlySummarizedExperiment}(x, ...)
}
\arguments{
\item{x}{\code{SummarizedExperiment} object}

\item{...}{unused arguments}
}
\value{
a data.frame object
}
\description{
create data.frame
}
\examples{
as.data.frame(se_simple)

}
